;**************************************************************************
; RPOT002.ASM
;
; PROGRAM: 	Demonstrator to show how to read POT position and scale result 
;		to reduce effects of component tolerence.
;
; DESCRIPTION: 	This program uses an RC timer to measure the osition of a joystick.
; 		The Pot needs to be calibrated, and code does a 
;		self calibrate.
;
;	Circuit 2: Mininal Component version 2 pots.
;
;		PA0 ---[ 100 k pot ]--- PA1
;                            ^
;			     | 
;                            &
;
;		PA3 ---[ 100 k pot ]--- PA2
;			 A   ^   B
;                            |
;			     +--------- PA4
;                            |
;			     |		    | |
;			     +--[ 220R ]----| |------ gnd
;					    | |
;	                                1uF capacitor
;
;
;	Serial Port Input:  RB0 - crash into routine.
;	Serial Port Output: RB1  
;
;

;
;		The design also uses an RS232 debug output, all in Hex
;
;		The format is: 
;		t:A=(A+B):meas<Range:output
;		t:72=95:72<95:00E6
;		t:72=95:72<95:00E6
;		t:72=95:72<95:
;		t:8B=B6:8B<B6:00E6
;		t:8B=B6:8B<B6:00E6
;		t:8B=B6:8B<B6:00E6
;		
;
;
;		RC is measured by applying a Logic 1 to PA3 or PA2.
;		Count up RCcounter until PA4 gets to logic 1
;		When the voltage gets to logic one on PA4 sample count
;
;		Rmin,Rnow,Rmax are counts. 
;
;		Convert the ratio of (Rnow-Rmin)/(Rmax-Rmin) to 0 to 5 min duration
;		using a successive approximation.
;		
;		Duration is in seconds.
;		Countdown-0x0101= seconds to go
;		* - timer running

; AUTHOR:  	Douglas Rice
; Copyright 2001
;
;	Crystal:	32kHz
;
;	I/O used:-
;	PORTA:0 RC measurement B1 
;	PORTA:1 RC measurement A1
;	PORTA:2 RC measurement B2 
;	PORTA:3 RC measurement A2
;	PORTA:4	RC discharge
;
;	
;	PORTB:0	Output A - Green  LED pulled low  to illuminate
;	PORTB:1	RS232 debug for Manufacture / Design Test 
;
;
;
	INCLUDE "p16F84.inc"       

;
;**************************************************************************

        LIST P=16F84, R=DEC
;        LIST P=16F628, R=DEC
	__idlocs  0x1234
	__config  _XT_OSC & _PWRTE_ON & _WDT_OFF

	cblock 	0x0c
		; variables used for context saving 
		w_temp
		status_temp
	endc

;--------------------------------------------------------------------------
; Sec 0. Build Constants - uncomment to include in the code.
;--------------------------------------------------------------------------
;

buildDEBUG	; if defined code easier to debug when single stepping.
buildCircuit2
;buildMacroRS232
;--------------------------------------------------------------------------
; Sec 1. Equates and Constants
;--------------------------------------------------------------------------

		; The General Purpose Registers start at the end of the 
		; Special Purpose Registers.

;--------------------------------------------------------------------------
; Sec 1.2 Button and LED 
;--------------------------------------------------------------------------

; PORTA binary values
; some port A bits are set in AWAI.inc

PORT_RS232	EQU	PORTA
RS232OUT	EQU	0

;--------------------------------------------------------------------------
; Sec 1.3 Table Macro
;--------------------------------------------------------------------------

TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary near " START   " "
	endif
	endm

;--------------------------------------------------------------------------
; Sec 2.0 Variables
;--------------------------------------------------------------------------
; Variables start 0x0C
;
	cblock 	
SRout 	; for RS232 routines
SRin	; Used in an interupt routine for serial Input
SRcnt		
SRloopCnt
SRinEvents
PORTBtemp

	;
	ENDC
	ifdef 	buildDEBUG	; Build to aid debug
PORT_TIME	EQU	PORTBtemp
	endif
	ifndef 	buildDEBUG	; Build to aid debug
PORT_TIME	EQU	PORTB
	endif


	cblock		; variables used in BCD.INC
BCDip3	
BCDip2	
BCDip1	
BCDip0	
BCDop0      ;equ     13              ; RAM Assignments
BCDop1      ;equ     14
BCDop2      ;equ     15
BCDop3      ;equ     16
BCDop4      ;equ     17

BCDcount  ;equ      18
BCDtemp   ;equ      19
	endc

;--------------------------------------------------------------------------
; Sec 3. Program Code
;--------------------------------------------------------------------------

	ORG     0       
	nop	
        GOTO    preStart

	ORG 	4
	GOTO	Intrtn	


;--------------------------------------------------------------------------
; Sec 3.1	Main Program Init Code
;--------------------------------------------------------------------------

preStart

	MOVLW	8
	MOVWF	SRcnt
	; start of serialIn
	; call	Intrtn;
	; end serialIn

	BSF	STATUS, RP0
	MOVLW	0xE1
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0

	movlw	0xff
	BCF   SRinEvents,0	; If serial input, then crash through output

	movlw	0x55
	call	SRtxChar
	movlw	0xAA
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar
	movlw	0x0D


	call	SRtxChar
	movlw	'T'
	call	SRtxChar
	movlw	'L'
	call	SRtxChar
	movlw	'O'
	call	SRtxChar
	movlw	'G'
	call	SRtxChar
	movlw	':'
	call	SRtxChar

	; Change Prescaler
	
CLRWDT
	; Configure Tmr 0
	; Set up prescaller for 8192 / 16 = 512 ticks perseconds.
	; or two timeouts per second
	; time out every 8 seconds
	; For Tmr0 0=/2, 1=/4, 2=/8,3=/16, 4=/32
	MOVLW	1 << INTEDG  | (0x0 + 2) 	

	BSF	STATUS,RP0
	MOVWF	OPTION_REG
	BCF	STATUS,RP0

Start

	; Enable Interupts
	MOVLW	1 << GIE | 1 << INTE
	MOVWF	INTCON

;	CALL	FLledOff	; debug 

	; Measure all 4 resistors
	CALL	RCmeasureR2
	CLRWDT
	CALL	SRlistMeasurements

	; calc pot position and display result

	CALL	MRcalcABdivC_A0A1
	CALL	BCDConvertResultToDecimal
	CALL	SRlistMeasurements1

	CALL	MRcalcABdivC_A2A3
	CALL	BCDConvertResultToDecimal
	CALL	SRlistMeasurements1

	CALL	SRlistMeasurements3

WaitForTimeout
	CLRWDT
	BTFSC   SRinEvents,0	; If serial input, then crash through output
	goto	MLprocSR

	BTFSS	INTCON,T0IF
	GOTO	WaitForTimeout
	; TMR0 timeout

	BCF	INTCON,T0IF
	goto Start

MLprocSR
	BCF   SRinEvents,0	; If serial input, then crash through output
	; Serial input process.

	movlw	0x0A
	call	SRtxCharLp

	movlw	0x0D
	call	SRtxCharLp

	movlw	'I'
	call	SRtxCharLp

	movlw	'P'
	call	SRtxCharLp

	movlw	':'
	call	SRtxCharLp

	movfw	SRin
	call	SRtxCharLp

	movfw	SRin
	andlw	0x7F
	addlw	-'F'
	BZ	MLprocSRf
	addlw	-'S'+'F'
	BZ	MLprocSRs

	goto 	Start


	BSF	STATUS,RP0
	; Set up prescaller for 8192 / 16 = 512 ticks perseconds.
	; or two timeouts per second
	; time out every 8 seconds
	; For Tmr0 0=/2, 1=/4, 2=/8,3=/16, 4=/32

MLprocSRf
	MOVLW	1 << INTEDG | (0x0 + 2 ) 	
	GOTO	MLprocSRset

MLprocSRs
	MOVLW	1 << INTEDG | (0x0 + 7 ) 	

MLprocSRset
	CLRWDT
	BSF	STATUS,RP0
	MOVWF	OPTION_REG
	BCF	STATUS,RP0
	movlw	0xFE
	movwf	TMR0
	goto 	Start


;--------------------------------------------------------------------------
; Sec 3.2	Main Program 
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Sec 4.10	RC measurment routine
;--------------------------------------------------------------------------
;				

;    GND --| |--[ 200R ]--+---[ R2 ]---- RA2
;	    C 	          |
;		          +---[ R3 ]---- RA3
;		          |
;		          +---RA4/TOCLKI
;
; R2 and R3 form the two sides of a pot.
;	---[ R2    R3 ]---
;                ^
;                |
;                +--------
;
; To measure CR enable either RA2 or RA3  as an output and RA4 as an input. 
;
; Set RA2 or RA3 to start charging C, use the input to see if it has got to logic 1.
;

	cblock
RCresultRm
RCresultR0
RCresultR1
RCresultR2
RCresultR3
RCresultR4
RCresultR5
	endc

RCmeasureR2
	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	CLRF	PORTA
	MOVLW	~( 1 << 0 )
	; count how long it takes to charge C
	CALL	RCmeas2
	MOVWF	RCresultR0


	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	CLRF	PORTA
	MOVLW	~( 1 << 1 )
	; count how long it takes to charge C
	CALL	RCmeas2
	MOVWF	RCresultR1


	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	CLRF	PORTA
	MOVLW	~( 1 << 2 )
	; count how long it takes to charge C
	CALL	RCmeas2
	MOVWF	RCresultR2
	
	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	CLRF	PORTA
	MOVLW	~( 1 << 3 )
	; count how long it takes to charge C
	CALL	RCmeas2
	MOVWF	RCresultR3

	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	CLRF	PORTA
	MOVLW	~( 1 << 4 )
	; count how long it takes to charge C
	CALL	RCmeas2
	MOVWF	RCresultR4

	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	;	leave RCresultR2 then measure R3
	CLRF	PORTA

	RETURN

RCmeas2
	
	GOTO $+1
	GOTO $+1
	GOTO $+1
	GOTO $+1
	GOTO $+1
	GOTO $+1
	GOTO $+1
	GOTO $+1
	CLRWDT

	CLRF	RCresultRm
	INCFSZ	RCresultRm,f

	TRIS	PORTA
	MOVLW	0xFF
	MOVWF	PORTA		; Set output high to start Measurement
RCmeasR2Loop	
	CLRWDT

	BTFSC	PORTA,4
	GOTO	RCmeasureR2End
	INCFSZ	RCresultRm,f
	GOTO 	RCmeasR2Loop
RCmeasureR2End
	DECFSZ	RCresultRm,f	; limit max
	MOVFW	RCresultRm 

	RETURN




	; You could leave RCresultR2 anduse its value
	; as the starting count for RCresultR2
	; to get the sum or R2 + R3

;MMnextTry    := VinRef; 
;MMoutNextTry := VoutRef; 
;MMitteration := 0;
;MMoutput     := 0;
;
;For MMprecision  := 0 to 8 do
;BEGIN
;  MMnextTry    := MMnextTry div 2;
;  MMoutNextTry := MMoutNextTry div 2;
;
;  ; Compare Vmeas against itteration
;
;  if ( MMmeas => ( MMitteration+MMnextTry )
;  BEGIN
;    MMitteration := MMitteration + MMnextTry;
;    MMoutput     := MMoutput + VoutNextTry
;  END
;END

	cblock
MMmeasA
MMitteration
MMnextTry
MMoutNextTry
MMoutputhA
MMoutputlA
MMprecision
	endc
		

MRcalcABdivC_A0A1
	; R1 / ( R1 + R0 )
	MOVFW	RCresultR1	; use R2 as measured vaule
	MOVWF	MMmeasA
	CLRC			; Add R2+R3 to give range
	MOVFW	RCresultR0
	ADDWF	RCresultR1,w
	MOVWF	MMnextTry
	GOTO 	MRcalcABdivC2

MRcalcABdivC_A2A3
	; R3 / ( R3 + R2 )
	MOVFW	RCresultR3	; use R2 as measured vaule
	MOVWF	MMmeasA
	CLRC			; Add R2+R3 to give range
	MOVFW	RCresultR2
	ADDWF	RCresultR3,w
	MOVWF	MMnextTry
	GOTO 	MRcalcABdivC2


MRcalcABdivT
	; R3 / ( R3 + R1 )
	MOVFW	RCresultR3	; use R2 as measured vaule
	MOVWF	MMmeasA
	CLRC			; Add R2+R3 to give range
	MOVFW	RCresultR1
	ADDWF	RCresultR3,w
	MOVWF	MMnextTry
	GOTO 	MRcalcABdivC2

MRcalcABdivC2
      
	; set to 270 degrees / 2 to save 16 bit arithmatic
	MOVLW	.270 / .2
	MOVWF	MMoutNextTry	; this is divided by 2

	CLRF	MMitteration
	CLRF	MMoutputhA
	CLRF	MMoutputlA
	movlw	8
	movwf	MMprecision
MMcalcLoop2	
	CLRWDT
	clrc
	rrf	MMnextTry,f
	;	MMrangeT needs to be 270 degrees which is > 255 so divide by 2 now
	; calc  if MMmeas > MMitteration+MMnextTry/2	
	clrc
	movfw	MMnextTry
	ADDWF	MMitteration,w
	SUBWF	MMmeasA,w	; w= f-w
	BNC	MMcalc2 
	BZ	MMcalc2 

	; work out new nextTry for meas
	movfw	MMnextTry
	addwf	MMitteration,f
	; Slave Calc the Timer value
	movfw	MMoutNextTry
	addwf	MMoutputlA,f
	BNC	MMcalc2 
	incfsz	MMoutputhA,f
MMcalc2
	clrc
	rrf	MMoutNextTry,f
	decfsz	MMprecision,f
	goto MMcalcLoop2

	; MMoutputlA and MMoutputhA

	RETURN



;--------------------------------------------------------------------------
; Sec 4.9	Flash LEDs
;--------------------------------------------------------------------------
FLwhichLed

	MOVFW	SRin

	; use the inverted format and add '2'
	ADDLW	-'1'
	SKPNZ
	GOTO	FLled1		; debug 
	ADDLW	-'2'+'1'
	SKPNZ
	GOTO	FLled2		; debug 
	ADDLW	-'3'+'2'
	SKPNZ
	GOTO	FLled3		; debug 
	ADDLW	-'0'+'3'
	SKPNZ
	CALL	FLledOff	; debug 
	RETURN


FLledOff
	BSF	STATUS, RP0
	MOVLW	0xE1
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0
	MOVLW	0x0C
	MOVWF	PORTB		; Set output high to start Measurement
	RETURN

FLled1
	BSF	STATUS, RP0
	MOVLW	0xE1
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0
	MOVLW	0x0C^4
	MOVWF	PORTB		; Set output high to start Measurement
	RETURN

FLled2
	BSF	STATUS, RP0
	MOVLW	0xE1
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0
	MOVLW	0x0C^2
	MOVWF	PORTB		; Set output high to start Measurement
	RETURN

FLled3
	BSF	STATUS, RP0
	MOVLW	0xE1
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0
	MOVLW	0x0C^8
	MOVWF	PORTB		; Set output high to start Measurement
	RETURN
		
;--------------------------------------------------------------------------
; Sec 4.9	Serial Out - Generic routine
;--------------------------------------------------------------------------
;				
;		@ 8192 instructions a second, 
;		for 1200 baud, each bit = 6.82666 instructions
;		
;		---_01234567P---- 
;		-> Start     Stop
;		The output signal is inverted by the driver transistor
;
;		8192/	instrns	cumulative	% er
;		1200
;
;		0.000		0	
;	start	6.827	7	7	102.54%
;	0	13.653	7	14	102.54%
;	1	20.480	7	21	102.54%
;	2	27.307	6	27	98.88%
;	3	34.133	7	34	99.61%
;	4	40.960	7	41	100.10%
;	5	47.787	7	48	100.45%
;	6	54.613	7	55	100.71%
;	7	61.440	6	61	99.28%
;	8	68.267	7	68	99.61%
;	end	75.093	7	75	99.88%
;					

;#Is the output inverted? 
;# Normally the UART OP is inverted by the Line driver
;# but as design does not uses a driver invert the bits
RS232opHigh	MACRO	
		BCF	PORT_RS232,RS232OUT
	endm

RS232opLow	MACRO	
		BSF	PORT_RS232,RS232OUT
	endm

RS232opBit	MACRO   bit
		BTFSC	SRout,bit		; 1 if false, 2 if skip
		GOTO	SRtxCharA#v(bit)	; 	t+1 instructions
		NOP			;t+2
		;BSF	PORT_RS232,RS232OUT		;t+3
		RS232opLow
		GOTO	SRtxCharA#v(bit)a	;t+4
SRtxCharA#v(bit)	;BCF	PORT_RS232,RS232OUT		;	t+3
		RS232opHigh

		NOP			;	t+4
		NOP			;	t+5
SRtxCharA#v(bit)a	NOP			;t+6	t+6
	endm

SRtxChar
	ifdef buildMacroRS232
		;BSF	PORT_RS232,RS232OUT		;		t=0
		RS232opLow
;		MOVLW	'U'
		MOVWF	SRout

		NOP			;		t+1
		NOP			;		t+2		
;		NOP			;		t+3
		; These take 4 instruction cyles
		; t=0
		NOP


		RS232opBit	0
		RS232opBit	1
		RS232opBit	2
		RS232opBit	3
		RS232opBit	4
		RS232opBit	5
		RS232opBit	6
		RS232opBit	7
		; Stop Bit
		RS232opHigh
;		BCF	PORT_RS232,RS232OUT

		NOP
;		NOP
;		NOP
;		NOP
;		NOP
;		NOP
		RETURN
	endif




;SRtxChar
SRtxCharLp
	cblock 	
SRtxTemp
SRtxCnt

	endc

	BTFSC   SRinEvents,0	; If serial input, then crash through output
	RETURN

	;
	;Output Start Bit
	;

	movwf	SRtxTemp
	RLF	SRtxTemp,f ; if using RB1 shift left

	comf	SRtxTemp,f
	BCF	PORTB,1
	MOVLW	~( 1 << 1 )
;	TRIS	PORTA
	BSF	STATUS, RP0
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0

	; 1200 baud output
	; move the char into SRtxtemp, it is destroyed.
	;Start Bit
	BSF	PORTB,1
	movlw	4
	movwf	SRtxCnt
	GOTO 	$+1
SRtxCharLp1
	; This outputs two bits at a time 
	; this takes 6.5 cycles each
	
	; After start bit, which is 1, then test each bit
	; I need to set PA:0 to same as LSB SRtxTemp
	; this compares PA:0 with SRtxTemp so see if it needs toggling
	; The xorwf PORTA,f causes the bit to be toggled if required.

	movfw	SRtxTemp
	xorwf	PORTB,W
	andlw	0x2
	xorwf	PORTB,f		;t+6,19
	RRF	SRtxTemp,f

	movfw	SRtxTemp
	xorwf	PORTB,W
	andlw	0x2
	RRF	SRtxTemp,f
	xorwf	PORTB,f		;t+11,24


	decfsz	SRtxCnt
	goto	SRtxCharLp1
	GOTO	$+1	
	GOTO	$+1	
	BCF	PORTB,1
	GOTO	$+1	
	GOTO	$+1	
	RETURN



SRwrtHexNibble	; currently only displays 0..9, A..F
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar

;--------------------------------------------------------------------------
; Sec 4.10	BCD routine	
;--------------------------------------------------------------------------

BCDConvertResultToDecimal

	; we arm to display first of all
	movfw	MMoutputlA
	movwf	BCDip0

	movfw	MMoutputhA
	movwf	BCDip1

	clrf	BCDip2
	clrf	BCDip3

	clrf    BCDop0
	clrf    BCDop1
	clrf    BCDop2
	clrf    BCDop3
	clrf    BCDop4


FRdisplayResult


#DEFINE	buildbcdBinDec24
	call    B2_BCD          ; After conversion the Decimal Number
	return





SRlistMeasurements

	MOVLW	0x0D
	CALL	SRtxChar	
	MOVLW	0x0A
	CALL	SRtxChar	
	MOVLW	'f'
	CALL	SRtxChar	
	MOVLW	'('
	CALL	SRtxChar	
	MOVLW	'"'
	CALL	SRtxChar	
	MOVLW	','
	CALL	SRtxChar	
	
	SWAPF	RCresultR0,w
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR0
	CALL	SRwrtHexNibble	
	MOVLW	','
	CALL	SRtxChar	
	
	SWAPF	RCresultR1,w
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR1
	CALL	SRwrtHexNibble	

	MOVLW	','
	CALL	SRtxChar	
	SWAPF	RCresultR2,w
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR2
	CALL	SRwrtHexNibble	

	MOVLW	','
	CALL	SRtxChar	
	SWAPF	RCresultR3,w
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR3
	CALL	SRwrtHexNibble	
	MOVLW	','
	CALL	SRtxChar	

	RETURN


SRlistMeasurements1
	MOVLW	':'
	CALL	SRtxChar	

	MOVLW	','
	CALL	SRtxChar	

	SWAPF	MMoutputhA,w
	CALL	SRwrtHexNibble	

	MOVFW	MMoutputhA
	CALL	SRwrtHexNibble	

	SWAPF	MMoutputlA,w
	CALL	SRwrtHexNibble	

	MOVFW	MMoutputlA
	CALL	SRwrtHexNibble	

	MOVLW	','
	CALL	SRtxChar	

	SWAPF	BCDop2,W
	CALL	SRwrtHexNibble	

	MOVFW	BCDop2
	CALL	SRwrtHexNibble	

	SWAPF	BCDop1,W
	CALL	SRwrtHexNibble	

	MOVFW	BCDop1
	CALL	SRwrtHexNibble	

	SWAPF	BCDop0,W
	CALL	SRwrtHexNibble	

	MOVFW	BCDop0
	CALL	SRwrtHexNibble	

	MOVLW	','
	CALL	SRtxChar	

	RETURN

SRlistMeasurements3

	MOVLW	','
	CALL	SRtxChar	
	MOVLW	'"'
	CALL	SRtxChar	
	MOVLW	')'
	CALL	SRtxChar	
	MOVLW	';'
	CALL	SRtxChar	

	RETURN
		

;--------------------------------------------------------------------------
; Sec 4.10	Binary to BCD -Procedure
;-------------------------------------------------------------------------
	include	"bcd.inc"


;--------------------------------------------------------------------------
; Sec 5.0	Interrupt Routines
;--------------------------------------------------------------------------

Intrtn	

		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		; Which Interupt ?
		; No Context Save in these routines.

		; RS232 1200 baud serial code 
		nop
		
TurboSRinA
		; for the 1200 baud serial input, each bit takes 6.83 cycles(8192/1200)
		; We need to sample 1.5 bits (  10.245 cycles in )
		;RS232 input pin must be RB0 ( or RB7 with mods )

TurboSRinA1
		RRF	PORTB,W	; shift LSB into Carry
		RRF	SRin,f
		goto	$+1	; wants 2 cycle delay
		DECFSZ	SRcnt,f
		GOTO	TurboSRinA1
		; now set up the counter for next time
		MOVLW	8
		MOVWF	SRcnt

		; The SRin needs to be inverted
		COMF	SRin,f

IntrtnEnding
		BSF	SRinEvents,0	; Set this bit to indicate a serial byte.

		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents

		BCF 	INTCON,INTF
		retfie                    ; return from interrupt
;		return


		BSF	SRinEvents,0	; Set this bit to indicate a serial byte.

CodeEnd		goto	CodeEnd



;--------------------------------------------------------------------------
; Program End
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; EEPROM data
;--------------------------------------------------------------------------
	ORG	0x2100
	cblock
LASTcont
	endc
        END

;--------------------------------------------------------------------------
; Serial output format
; The data is wrapped in a function that can be included in JavaScript, 
; so you can use a Web page to process the output.
;
; The data is Comma Separated Values
;--------------------------------------------------------------------------
;/0,1 ,2 ,3 ,4 ,5,6   ,7       ,8   ,9     ,10,11);
;   Counts     :270*A1/(A1+A0):270*A3/(A2+A3)
;/  A0,A1,A2,A3,:,hex ,DecA0A1:,Hex ,DecA2A3,,

f(",11,DD,94,5D,:,00FA,000250,:,0067,000103,,");
f(",2A,C2,94,5D,:,00DD,000221,:,0067,000103,,");
f(",31,BC,95,5D,:,00D7,000215,:,0067,000103,,");
f(",6A,80,94,5D,:,0094,000148,:,0067,000103,,");
f(",78,77,95,5D,:,0083,000131,:,0067,000103,,");
f(",76,8C,95,5D,:,010A,000266,:,0067,000103,,");
f(",CC,22,95,5D,:,0028,000040,:,0067,000103,,");

